@@ -349,6 +349,7 @@ void osmo_pcap_conn_restart_trace(struct osmo_pcap_conn *conn)
 
 	conn->wrf = osmo_pcap_wr_file_alloc(conn, conn);
 	osmo_pcap_wr_file_set_flush_completed_cb(conn->wrf, osmo_pcap_wr_file_flush_completed_cb);
+	osmo_pcap_wr_file_set_write_queue_max_length(conn->wrf, conn->server->file_wr_queue_max_length);
 	rc = osmo_pcap_wr_file_open(conn->wrf, curr_filename, conn->server->permission_mask);
 	talloc_free(curr_filename);
 	if (rc < 0)
@@ -543,12 +544,20 @@ int osmo_pcap_conn_process_data(struct osmo_pcap_conn *conn, struct msgb *msg)
 	if (!check_restart_pcap_max_size(conn, msgb_length(msg)))
 		check_restart_pcap_localtime(conn, now);
 
+	/* Since the checks above may have closed and re-opened (and could have
+	 * failed), check again whether new wrf is ready... */
+	if (!conn->wrf) {
+		LOGP(DSERVER, LOGL_ERROR, "No file is open. close connection.\n");
+		return -1;
+	}
+
 	talloc_steal(conn->wrf, msg);
 	rc = osmo_pcap_wr_file_write_msgb(conn->wrf, msg);
 	if (rc < 0) {
 		LOGP(DSERVER, LOGL_ERROR, "%s: Failed writing to file\n", conn->name);
-		/* msgb will be freed by caller */
-		return -1;
+		msgb_free(msg);
+		/* Avoid closing conn. msgb won't be freed by caller */
+		return 0;
 	}
 	update_last_write(conn, now);
 	/* msgb is now owned by conn->wrf. */
@@ -570,6 +579,7 @@ struct osmo_pcap_server *osmo_pcap_server_alloc(void *ctx)
 	psrv->max_size = 1073741824; /* 1024^3, 1GB **/
 	psrv->max_size_enabled = true;
 	psrv->max_snaplen = DEFAULT_SNAPLEN;
+	psrv->file_wr_queue_max_length = PCAP_SERVER_FILE_WRQUEUE_MAX_LEN;
 	/* By default rotate daily: */
 	psrv->rotate_localtime.enabled = true;
 	psrv->rotate_localtime.intv = TIME_INTERVAL_DAY;
